;;; ******************************************************************
;;;
;;;     CADAM_to_ACAD... rorganisation des objets par calque
;;;
;;;     Christian REB, le 14 Dcembre 2012 pour KUHN, Saverne


;;; ----------------------------------------
;;;	Fonction principale
;;; ----------------------------------------

(defun C:CADAM_to_ACAD()

	;;; --------------------------------
	;;;	Variables PROG
	;;; --------------------------------

	;;;	  NOM du CALQUE   /      COULEUR      /      TYPE DE LIGNE       /     EPAISSEUR

	(setq	C_AXE	"0-AXE"      C_AXE_COUL   "1"    C_AXE_TL   "axes"         C_AXE_EP   "0.13"
 		C_COTE	"0-COTE"     C_COTE_COUL  "4"    C_COTE_TL  "continuous"   C_COTE_EP  "0.25"
		C_FORT	"0-FORT"     C_FORT_COUL  "5"    C_FORT_TL  "continuous"   C_FORT_EP  "0.50"
		C_FIN	"0-FIN"      C_FIN_COUL   "1"    C_FIN_TL   "continuous"   C_FIN_EP   "0.13"
		C_CACHE "0-CACHE"    C_CACHE_COUL "3"    C_CACHE_TL "cache"        C_CACHE_EP "0.25"
		C_HACH	"0-HACH"     C_HACH_COUL  "6"    C_HACH_TL  "continuous"   C_HACH_EP  "0.13"
		C_TEXT	"0-TEXT"     C_TEXT_COUL  "3"    C_TEXT_TL  "continuous"   C_TEXT_EP  "0.25"
		C_POINT "0-POINT"
	);setq

	;;; --------------------------------
	;;;	Initialise environnement
	;;; --------------------------------

	(setvar  "CMDECHO"	0   ) ;;; affichage des message de commandes
	(setvar  "EXPERT"	5   ) ;;; suppression de messages "INUTILES !!!"
	(setvar  "MSLTSCALE"	1   ) ;;; pour un affichage plus ou moins corrrect dans l'ONGLET OBJET !!!
	(setvar  "PSLTSCALE"	1   ) ;;; active l'chelle automatique dans les prsentations
	(setvar  "MEASUREMENT"	1   ) ;;; Active le mode METRIQUE
	(setvar  "MEASUREINIT" 	1   ) ;;; MODE METRIQUE
	(setvar	 "INSUNITS"	4   ) ;;; Units du dessin = MM (5= CM / 6= M)
	(setvar  "LTSCALE"	0.4 ) ;;; echelle des types de ligne
	(setvar  "PDMODE"      35   ) ;;; modle de point : masque les objets points ( 0 = pixel )

	(command "_-scalelistedit" "R" "OUI" "Q")	;;; rinitialise les chelles mtriques
	(command "_-layout" "" "MODEL")			;;; place dans l'onglet OBJET 
	(command "_-linetype" "CH" "*" "ACADISO.LIN" "")   ;;; recharge les type de ligne format METRIQUE

	;;; --------------------------------
	;;;	Cration des calques
	;;; --------------------------------

	(command "_-layer"

		 "L" "0" "AC" "0" "E" "0" "CO" "7" "" "TL" "continuous" "" "ep" "0.25" ""  	;;; initialise le calque 0

		 "AC" "*" "L" "*" "D" "*"	;;; ACTIVE, LIBERE, DEVERROUILLE tous les calques

		 "E" C_AXE	"CO" C_AXE_COUL ""	"TL" C_AXE_TL ""	"EP" C_AXE_EP ""
		 "E" C_COTE	"CO" C_COTE_COUL ""	"TL" C_COTE_TL ""	"EP" C_COTE_EP ""
		 "E" C_FORT	"CO" C_FORT_COUL ""	"TL" C_FORT_TL ""	"EP" C_FORT_EP ""
		 "E" C_FIN	"CO" C_FIN_COUL ""	"TL" C_FIN_TL ""	"EP" C_FIN_EP ""
		 "E" C_CACHE	"CO" C_CACHE_COUL ""	"TL" C_CACHE_TL ""	"EP" C_CACHE_EP ""
		 "E" C_HACH	"CO" C_HACH_COUL ""	"TL" C_HACH_TL ""	"EP" C_HACH_EP ""
		 "E" C_TEXT	"CO" C_TEXT_COUL ""	"TL" C_TEXT_TL ""	"EP" C_TEXT_EP ""
		 "E" C_POINT
		 "E" "OVERLAY_MAP"

		 ""
	);command LAYER

	;;; -------------------------------------
	;;;	prparation de certains calques
	;;; -------------------------------------

	(setvar "CLAYER"	"0") 			;;; claque courant

	(command "_-laydel" "N" "OVERLAY_MAP" "" "oui") ;;; supprime le calque OVERLAY_MAP

	(command "CHPROP" "_ALL" "" "CA" "0" "")	;;; place tous les objets dans le calque 0


	;;; -------------------------------------
	;;;	cration FILTRES OBJETS
	;;; -------------------------------------

	;;;                                          TYPE DE LIGNE       TYPE OBJET

	(setq	OBJ_AXE		(ssget "X"	' ( (6 . "C_CENTER_L") 			  ) ) ;;; filtre les traits axes
		OBJ_COTE	(ssget "X"	' ( 			(0 . "DIMENSION") ) ) ;;; filtre les cotes
		OBJ_FORT	(ssget "X"	' ( (6 . "C_SOLID_M")			  ) ) ;;; filtre les traits forts
		OBJ_FIN		(ssget "X"	' ( (6 . "C_SOLID_L")			  ) ) ;;; filtre les traits fins
		OBJ_CACHE	(ssget "X"	' ( (6 . "C_DASHED_L")			  ) ) ;;; filtre les traits cachs
		OBJ_HACH	(ssget "X"	' ( 			(0 . "HATCH")	  ) ) ;;; filtre les hachures
		OBJ_TEXT	(ssget "X"	' ( 			(0 . "TEXT") 	  ) ) ;;; filtre les textes
		OBJ_MTEXT	(ssget "X"	' ( 			(0 . "MTEXT") 	  ) ) ;;; filtre les textes
		OBJ_POINT	(ssget "X"	' ( 			(0 . "POINT")	  ) ) ;;; filtre les points
	);setq
		
	;;; -------------------------------------
	;;;	change les objets de calque
	;;; -------------------------------------
	
	;;; ---	Calque AXE ---

	(if	OBJ_AXE
		(command "CHPROP" OBJ_AXE "" "CA" C_AXE "")
	);if

	;;; ---	Calque COTE ---

	(if	OBJ_COTE
		(command "CHPROP" OBJ_COTE "" "CA" C_COTE "")
	);if

	;;; ---	Calque FORT ---

	(if	OBJ_FORT
		(command "CHPROP" OBJ_FORT "" "CA" C_FORT "")
	);if

	;;; ---	Calque FIN ---

	(if	OBJ_FIN
		(command "CHPROP" OBJ_FIN "" "CA" C_FIN "")
	);if

	;;; ---	Calque CACHE ---

	(if	OBJ_CACHE
		(command "CHPROP" OBJ_CACHE "" "CA" C_CACHE "")
	);if

	;;; ---	Calque HACH ---

	(if	OBJ_HACH
		(command "CHPROP" OBJ_HACH "" "CA" C_HACH "")
	);if

	;;; ---	Calque TEXTE ---

	(if	OBJ_TEXT
		(command "CHPROP" OBJ_TEXT "" "CA" C_TEXT "")
	);if

	(if	OBJ_MTEXT
		(command "CHPROP" OBJ_MTEXT "" "CA" C_TEXT "")
	);if

	;;; ---	Calque POINT ---

	(if	OBJ_POINT
		(command "CHPROP" OBJ_POINT "" "CA" C_POINT "")
	);if


	;;; ---------------------------------------------------
	;;; change TOUS les OBJETS en DUCALQUE
	;;; ---------------------------------------------------

	(command "chprop" "_ALL" "" "CO" "DUCALQUE" "TL" "DUCALQUE" "EP" "DUCALQUE" "")	  ;;; tous les objets en DUCALQUE

	(command "_-layer" "G" C_POINT "")						  ;;; GELE le calque des POINTS

	(command "_audit" "_Y")								  ;;; CONTROLE	

	(command "_zoom" "_E")								  ;;; ZOOM etendu

(princ)
);defun


;;; ----------------------------------------
;;;	Fonction Cration PRESENTATION
;;; ----------------------------------------

(defun C:CADRE()

	(prompt "\nSlectionner les objets de la PIECE uniquement !!!!")

	(setq	ENT_PIECE	(ssget) )			;;; selection de la pice

	(command "_-block" "PIECE_OBJET" "0,0" ENT_PIECE "")	;;; cration du bloc de la piece

	(command "_cutclip" (ssget "x") "")			;;; coupe le cadre et cartouche

	(command "_-layout" "N" "CADRE" "_-layout" "E" "CADRE") ;;; cre une prsentation CADRE

	(command "_pasteclip" "0,0" "" "_zoom" "_e")		;;; COLLE le cadre et le cartouche

	(command "_-layer" "E" z_fenetre	"CO" "magenta" ""	"ep" "0.25" ""		"T" "A" ""	"") ;;; cre calque Z_FENETRE

	(prompt  "\n1er et 2e point de la fentre...")
	(command "_-vports" pause pause)			;;; cration d'une fentre

	(command "_mspace" "_-insert" "*piece_objet" "0,0" "1" "0" "_zoom" "_e") ;;; insertion de la pice

	(setvar "CLAYER" "0")					;;; rend courant le calque 0

	(command "_-purge" "TOUT" "*" "n" "_-purge" "TOUT" "*" "n" "_-purge" "R" "*" "n") ;;; PURGE le dessin

	(command "_audit" "_Y")								  ;;; CONTROLE	

);defun CADRE

;;; -------------------------------- FIN PROG -----------------------------------

(prompt "\nTaper    CADAM_to_ACAD    pour rorganiser les objets par calque...")
(prompt "\nTaper    CADRE            pour crer la PRESENTATION...")

(princ)








