;;; INSTOPO (gile)
;;; Insre le bloc "PointBloc" sur les points dcrit dans un fichier ascii (.txt .csv ou autre)
;;;
;;; Modification : possibilit d'insrer un point 02/12/2009

(defun c:instopo (/        *error*  makeblock         filename tmp
                  file     layers   clay     ptlay    blklay   data-sep
                  dec-sep  mat-p    mat      alt      scl      point
                  bloc     dcl_id   space    line     coords   matric
                  insert
                 )

  (vl-load-com)
  (or *acdoc*
      (setq *acdoc* (vla-get-ActiveDocument (vlax-get-acad-object)))
  )
  (or *blocks* (setq *blocks* (vla-get-Blocks *acdoc*)))

  ;;---------------------------------------------------;;

  (defun *error* (msg)
    (or (= msg "Fonction annule")
        (princ (strcat "Erreur: " msg))
    )
    (and file (close file))
    (vla-EndUndoMark *acdoc*)
    (princ)
  )

  ;;---------------------------------------------------;;

  ;; MakeBlock
  ;; Cre le bloc PointBloc
  (defun makeblock (/ block)
    (vl-load-com)
    (setq block (vla-add *blocks*
                         (vlax-3d-point '(0. 0. 0.))
                         "TCPOINT"
                )
    )
    (vla-put-Layer
      (vla-addPoint block (vlax-3d-point '(0. 0. 0.)))
      "0"
    )
    (vla-put-Layer
      (vla-addAttribute
        block
        2.5
        acAttributeModePreset
        ""
        (vlax-3d-point '(1. 0.5 0.))
        "MAT"
        ""
      )
      "0"
    )
    (vla-put-Layer
      (vla-addAttribute
        block
        2.5
        acAttributeModePreset
        ""
        (vlax-3d-point '(1. -3. 0.))
        "ALT"
        ""
      )
      "0"
    )
  )

  ;;---------------------------------------------------;;

  (if (setq filename (getfiled "Slectionner un fichier point"
                               ""
                               "txt;csv;*"
                               0
                     )
      )
    (progn
      ;; Cration du fichier DCL temporaire
      (setq tmp  (vl-filename-mktemp "Tmp.dcl")
            file (open tmp "w")
      )
      (write-line
        "InsTopo
	:dialog{label=\"InsTopo\";
	:boxed_row{label=\"Format du fichier\";
	:boxed_radio_column{label=\"Sparateur de donnes\";key=\"data-sep\";
	:radio_button{label=\"Virgule\";key=\"44\";value =\"1\";}
	:radio_button{label=\"Point-virgule\";key =\"59\";}
	:radio_button{label=\"Espace\";key=\"32\";}
	:radio_button{label=\"Tabulation\";key =\"9\";}}
	:column{
	:boxed_radio_column{label=\"Sparateur dcimal\";key=\"dec-sep\";
	:radio_button{label=\"Virgule\";key =\"com\";}
	:radio_button{label=\"Point\";key=\"dot\";value=\"1\";}}
	:boxed_radio_column{label=\"Matricule\";key=\"mat-p\";
	:radio_button{label=\"Prsent\";key=\"present\";value=\"1\";}
	:radio_button{label=\"Absent\";key=\"absent\";}}}}
	:boxed_column{label=\"Point\";
	:row{
	:toggle{label=\"Insrer des points\";key=\"point\";value=\"1\";}
	:popup_list{label=\"Calque\";key =\"ptlay\";edit_width = 24;}}
	spacer;}
	:boxed_column{label=\"Bloc\";
	:toggle{label=\"Insrer de blocs\";key=\"bloc\";value=\"1\";}
	:row{
	:boxed_column{label=\"Attributs\";
	:toggle{label=\"Matricule\";key=\"mat\";value=\"1\";}
	:toggle{label=\"Altitude\";key=\"alt\";value=\"1\";}}
	spacer;
	:column{spacer;
	:popup_list{label=\"Calque\";key =\"blklay\";edit_width = 24;}
	:edit_box{label=\"Echelle  \";key=\"scl\";value=\"1.0\";fixed_width=true;}
	spacer;}}}
	ok_cancel;}"
        file
      )
      (close file)

      (vlax-for l (vla-get-Layers *acdoc*)
        (setq layers (cons (vla-get-Name l) layers))
      )
      (setq layers   (vl-sort layers '<)
            clay     (getvar "CLAYER")
            ptlay    clay
            blklay   clay
            data-sep "44"
            dec-sep  "dot"
            mat-p    "present"
            mat      "1"
            alt      "1"
            scl      1.0
            point    T
            bloc     T
      )
      (setq dcl_id (load_dialog tmp))
      (if (not (new_dialog "InsTopo" dcl_id))
        (exit)
      )
      (start_list "ptlay")
      (mapcar 'add_list layers)
      (end_list)
      (start_list "blklay")
      (mapcar 'add_list layers)
      (end_list)
      (set_tile "ptlay" (itoa (vl-position clay layers)))
      (set_tile "blklay" (itoa (vl-position clay layers)))
      (foreach k (list "data-sep" "dec-sep" "mat-p" "mat" "alt")
        (action_tile k "(set (read $key) $value)")
      )
      (action_tile
        "point"
        "(if (= \"1\" $value)
          (progn
            (setq point T)
            (mode_tile \"ptlay\" 0)
          )
          (progn
            (setq point nil)
            (mode_tile \"ptlay\" 1)
          )
        )"
      )
      (action_tile
        "bloc"
        "(if (= \"1\" $value)
          (progn
            (setq bloc T)
            (mode_tile \"blklay\" 0)
            (mode_tile \"mat\" 0)
            (mode_tile \"alt\" 0)
            (mode_tile \"scl\" 0)
          )
          (progn
            (setq bloc nil)
            (mode_tile \"blklay\" 1)
            (mode_tile \"mat\" 1)
            (mode_tile \"alt\" 1)
            (mode_tile \"scl\" 1)
          )
        )"
      )
      (action_tile "ptlay" "(setq ptlay (nth (atoi $value) layers))")
      (action_tile "blklay" "(setq blklay (nth (atoi $value) layers))")
      (action_tile
        "scl"
        "(setq scl (distof $value))
	(while (or (not scl) (<= scl 0))
	(alert \"Ncessite un nombre rel strictement positif\")
	(setq scl 1.0)
	(set_tile \"scl\" \"1.0\")
	(mode_tile \"scl\" 2))"
      )
      (action_tile "cancel" "(setq data-sep nil) (done_dialog 0)")
      (action_tile "accept" "(done_dialog 1)")
      (start_dialog)
      (unload_dialog dcl_id)
      (vl-file-delete tmp)
      (if data-sep
        (progn
          (vla-StartUndoMark *acdoc*)
          (and (vl-catch-all-error-p
                 (vl-catch-all-apply
                   'vla-item
                   (list *blocks* "TCPOINT")
                 )
               )
               (makeblock)
          )
          (setq space (vla-get-ModelSpace *acdoc*)
                file  (open filename "r")
          )
          (while (setq line (read-line file))
            (setq coords (str2lst line (chr (atoi data-sep))))
            (if (= mat-p "present")
              (setq matric (car coords)
                    coords (cdr coords)
              )
              (setq matric nil)
            )
            (if (= dec-sep "com")
              (setq
                coords (mapcar
                         '(lambda (x)
                            (read (vl-string-translate "," "." x))
                          )
                         coords
                       )
              )
              (setq coords (mapcar 'read coords))
            )
            (setq coords (list (car coords)
                               (cadr coords)
                               (cond ((caddr coords))
                                     (T 0.0)
                               )
                         )
            )
            (if (vl-every 'numberp coords)
              (progn
                (if point
                  (vla-put-Layer (vla-AddPoint space (vlax-3d-point coords))
                                 ptlay
                  )
                )
                (if bloc
                  (progn
                    (setq
                      insert
                       (vla-InsertBlock
                         space
                         (vlax-3d-point coords)
                         "TCPOINT"
                         scl
                         scl
                         scl
                         0.0
                       )
                    )
                    (vla-put-Layer insert blklay)
                    (foreach att (vlax-invoke insert 'getAttributes)
                      (if
                        (and
                          (= (vla-get-TagString att) "MAT")
                          matric
                          (= mat "1")
                        )
                         (vla-put-TextString att matric)
                      )
                      (if
                        (and
                          (= (vla-get-TagString att) "ALT")
                          (= alt "1")
                        )
                         (vla-put-TextString att (rtos (caddr coords)))
                      )
                    )
                  )
                )
              )
            )
          )
          (close file)
          (vla-EndUndoMark *acdoc*)
        )
      )
    )
  )
  (princ)
)

;; str2lst
;; Transforme un chaine avec sparateur en liste de chaines
;;
;; Arguments
;; str : la chaine  transformer en liste
;; sep : le sparateur
;;
;; Exemples
;; (str2lst "a b c" " ") -> ("a" "b" "c")
;; (str2lst "1,2,3" ",") -> ("1" "2" "3")
;; (mapcar 'read (str2lst "1,2,3" ",")) -> (1 2 3)

(defun str2lst (str sep / pos)
  (if (setq pos (vl-string-search sep str))
    (cons (substr str 1 pos)
          (str2lst (substr str (+ (strlen sep) pos 1)) sep)
    )
    (list str)
  )
)