
;; 
;; Ci-joint un petit lisp trouv sur le net et amlior par mes soins 
;; permettant dexporter dans le fichier LINETYPE_EXPORT.LIN (localis dans le rpertoire du dessin) :
;; 
;; un type de ligne indiqu par loprateur : commande TO-LIN,
;; 
;; tous les types de ligne dfinis dans le dessin : commande ALL-TO-LIN.
;; 
;; Les types de ligne complexe sont aussi exports. 
;; Dans ce cas, si le fichier shp nest pas trouv dans les rpertoires de recherches dAutoCAD, 
;; il sera ncessaire de modifier le point lin pour le faire pointer vers un shp quivalent.
;; 
;; Programme original de Tee Square Graphics ameliore par ODU (GIS) 
;; 


;|

ExportLineType.LSP -- (c) Tee Square Graphics

	TO-LIN is a useful AutoLISP routine that extracts parameters for
	unknown LineTypes in a drawing, and creates entries in a new LineType
	definition file, LINETYPE_EXPORT.LIN. After extraction, the LineType definitions
	may be moved to ACAD.LIN or any other *.LIN file desired by the user.

	This version of TO-LIN.LSP functions fully with simple LineTypes, and
	Complex LineTypes composed of linear elements and Text objects. Because
	of difficulty in extracting Shape data from shape definition (*.shx)
	files, the user may, for the time being, have to supply the appropriate
	name for the Shape represented by {Shape #nnn} in NEWLT.LIN, in cases
	where an associated Shape Source File (*.shp) is unavailable.

	ExportLineTypes export all the line types defined in the current drawing

|;

(defun trim (x / i)
  (setq i (strlen x))
  (while (= (substr x i) "0")
    (setq i (1- i) x (substr x 1 i))
  )
  (if (= (substr x i) ".")
    (substr x 1 (1- i))
    x
  )
)

(defun BATCH-TO-LIN (ltname outFile / tblent flag tblist i
		     desc   alist  acode  value	 rot	shpno  shxfl
		     shpfl  inf	   dat	  n	 shpnm	flg    txt
		     sty
		    )
  (if (setq tblent (tblobjname "ltype" ltname))
    (progn
      (setq tblist (entget tblent) i 1 desc "A,")
      (setvar "luprec" 8)
      (setvar "auprec" 8)
      (write-line
	(strcat	"*"
		(cdr (assoc 2 tblist))
		","
		(cdr (assoc 3 tblist))
	)
	outFile
      )
      (while (< i (length tblist))
	(setq alist (nth i tblist)
	      acode (car alist)
	      value (cdr alist)
	)
	(cond ((= acode 49)
	       (setq desc (strcat desc (trim (rtos value 2 8)) ","))
	      )
	      ((= acode 74)
	       (setq flag (if (= (logand value 4) 4)
			    T
			    nil
			  )
		     rot  (if (= (logand value 1) 1)
			    "a"
			    "r"
			  )
	       )
	      )
	      ((= acode 75) (setq shpno (itoa value)))
	      ((= acode 340)
	       (if flag
		 (progn
		   (setq shxfl (cdr (assoc 3 (entget value)))
			 shpfl (strcat (substr shxfl 1 (- (strlen shxfl) 3))
				       "shp"
			       )
		   )
		   (if (setq inf (findfile shpfl))
		     (progn
		       (setq inf (open inf "r"))
		       (while (setq dat (read-line inf))
			 (if (wcmatch dat (strcat "`*" shpno "*"))
			   (progn
			     (setq n 1)
			     (repeat 2
			       (while (/= (substr dat n 1) ",")
				 (setq n (1+ n))
			       )
			       (setq n (1+ n))
			     )
			     (setq shpnm (substr dat n))
			   )
			 )
		       )
		       (close inf)
		     )
		   )
		 )
	       )
	       (setq flg  flag
		     txt  (if flag
			    (if	shpnm
			      shpnm
			      (strcat "{Shape #" shpno "}")
			    )
			    (strcat "\""
				    (cdr (assoc 9 (member alist tblist)))
				    "\""
			    )
			  )
		     sty  (if flag
			    (cdr (assoc 3 (entget value)))
			    (cdr (assoc 2 (entget value)))
			  )
		     desc (strcat
			    desc
			    "\n["
			    txt
			    ","
			    sty
			    ",s="
			    (trim (rtos (cdr (nth (1+ i) tblist)) 2 8))
			    ","
			    rot
			    "="
			    (trim (angtos (cdr (nth (+ i 2) tblist)) 0 8))
			    ",x="
			    (trim (rtos (cdr (nth (+ i 3) tblist)) 2 8))
			    ",y="
			    (trim (rtos (cdr (nth (+ i 4) tblist)) 2 8))
			    "],\n"
			  )
		     i	  (+ i 4)
	       )
	      )
	      (T nil)
	)
	(setq i (1+ i))
      )
      ;; while
      (write-line (substr desc 1 (1- (strlen desc))) outFile)
      (write-line " " outFile)
      (if (and flg (not shpnm))
	(princ
	  (strcat
	    "\nType de ligne " ltname " ajout  LINETYPE_EXPORT.LIN.\n"
	    "Il s'agit d'un type de ligne complexe utilisant un ou plusieurs fichiers shp\n"
	    "pour lequel aucun fichier source (*.shp) n'a t trouv, et\n"
	    "dont il faudra remplacer le(s) nom(s) entre les caractres { }.")
	)
	(princ (strcat "\nType de ligne "
		       ltname
		       " ajout  LINETYPE_EXPORT.LIN.\n"
	       )
	)
      )
    )
    (princ (strcat "\nType de ligne " ltname " inconnu !"))
  )
)



(defun C:ALL-TO-LIN (/ v1 ltName bRewind filePath outFile)
  (setq bRewind T)
  (setq filePath (findfile "LINETYPE_EXPORT.LIN")
	outFile (open (if filePath filePath (strcat (getvar "DWGPREFIX") "LINETYPE_EXPORT.LIN")) (if filePath "a" "w"))
  )
  (if (null filePath)
    (progn
      (write-line ";;" outFile)
      (write-line ";; ExportLineTypes.LSP : export des types de lignes" outFile)
      (write-line ";; dfinis dans un dessin." outFile)
      (write-line ";;" outFile)
      (write-line ";; ExportLineTypes.LSP (c) 2013 Tee Square Graphics / ODU (GIS) " outFile)
      (write-line ";;\n" outFile)
    )
  )

  (while (setq v1 (tblnext "LTYPE" bRewind))
    (setq ltName (cdr (assoc 2 v1)) bRewind nil)
    (BATCH-TO-LIN ltName outFile)
  )
  
  (close outFile)
  (princ)
)


(defun C:TO-LIN (/ ltName filePath outFile)
  (setq ltName (getstring "\nType de ligne  exporter : "))
  (if (and ltName (/= ltName ""))
    (progn
      (setq filePath (findfile "LINETYPE_EXPORT.LIN")
	    outFile (open (if filePath filePath (strcat (getvar "DWGPREFIX") "LINETYPE_EXPORT.LIN")) (if filePath "a" "w"))
      )
      (if (null filePath)
	(progn
	  (write-line ";;" outFile)
	  (write-line ";; ExportLineTypes.LSP : export des types de lignes" outFile)
	  (write-line ";; dfinis dans un dessin." outFile)
	  (write-line ";;" outFile)
	  (write-line ";; ExportLineTypes.LSP (c) 2013 Tee Square Graphics / ODU (GIS) " outFile)
	  (write-line ";;\n" outFile)
        )
      )

      (BATCH-TO-LIN ltName outFile)
  
      (close outFile)
    )
  )
  (princ)
) 

